% good ANN combos, based on goodPredictorsMore.pdf
Ca = [      1 3 4 5 8 0 0 ; 1 2 4 5 8 0 0 ];
Ca = [ Ca ; 1 2 4 6:8   0 ; 1:4   6 8   0 ];
Ca = [ Ca ; 1:5   7:8     ; 1:2   4:8     ];
Nc = length(Ca(:,1));

if exist('absoluteErrors.mat')~=2

  toyr = 3600*12*365e-5/5; % 5 day sum of micromolC/m2/s -> kgC/ha/yr
  
  global Ta Prod EVI PAR NEE;  PAR = max(0,PAR);
  
  load allData PAR NEE Ta r*
  
  load Tsoil.mat Tsoil
  
  load ../Data/antecedentPrecipPredictor.mat aPp
  
  %         1   2   3    4     5     6   7    8
  X     = [ PAR Ta rBLUE rNIR rRED rSWIR aPp Tsoil ];
  y     = NEE;
  
  Nt    = length(X(:,1));
  Np    = length(X(1,:));
  Nmc   = 100;
  Ny    = 1e4; % number of "annnual" means derived
  
  opts = optimset('display','off');  rng(13456465)

  % lambda, Topt, PAR, alpha, beta, Tlow. Start from the HF values in
  % Mahadevan et al.'s Table 2
  p0 = [ .127 20 570 .271 .25 5 ];

  load allData Prod EVI

  % get Nmc VPRM errors
  Ta0   =   Ta;
  Prod0 = Prod;
  EVI0  =  EVI; 
  PAR0  =  PAR;
  NEE0  =  NEE;
  Myv   = zeros(Nmc,Ny);
  Mdv   = zeros(Nmc,Ny);
  Erv   = zeros(Nmc,Ny);
  imc   = 0;
  while imc<Nmc
    kt   = randperm(Nt,round(Nt*.5)); % training set
    kv   = find(~ismember(1:Nt,kt )); % validation set
    Nv   = length(kv);
    Prod = Prod0(kt);
    Ta   = Ta0(kt);
    EVI  = EVI0(kt); 
    PAR  = PAR0(kt);
    NEE  = NEE0(kt);
    [p,~,e] = fminsearch(@dNEE,p0,opts); 
    if e==1
      yht = NEEnonLinReg(p,X(kv,1:6));
      if max(abs(yht))<60
	imc = imc+1;
	for iy = 1:Ny
	  ky          = randperm(Nv,120); % a random "5 days"
	  Myv(imc,iy) = sum( yht( ky ))*toyr;
	  Mdv(imc,iy) = sum(y(kv( ky)))*toyr;
	  Erv(imc,iy) = Myv(imc,iy) - Mdv(imc,iy);
	end
	disp(imc)
      end
    end
  end
  
  % get "annual" mean errors
  for i = 1:Nmc
    e = Erv(i,:);
    Erv(i,1) = sqrt(e*e'/Ny);
  end
  Erv = Erv(:,1);
  Myv = mean(Myv,2);
  Mdv = mean(Mdv,2);
  
  clear Ta* Prod* EVI* PAR* NEE*
  
  % get top 2 ANN errors for Np=[5,7] based on goodPredictorsMore.pdf
  Myn   = zeros(Nmc,Ny,Nc);
  Mdn   = zeros(Nmc,Ny,Nc);
  Ern   = zeros(Nmc,Ny,Nc);
  for i = 1:Nc % loop on top-2 combos
    imc   = 0;
    while imc<Nmc
      kt   = randperm(Nt,round(Nt*.5)); % training set
      kv   = find(~ismember(1:Nt,kt )); % validation set
      clear net
      net = feedforwardnet([6 5 4]);
      net.divideFcn = '';
      net.trainParam.showWindow = false;
      j = find(Ca(i,:));
      j = Ca(i,j);
      net = train(net,X(kt,j)',y(kt)');
      yht = net(X(kv,j)')';
      if max(abs(yht))<60
	imc = imc+1;
	for iy = 1:Ny
	  ky            = randperm(Nv,120); % a random "5 days"
	  Myn(imc,iy,i) = sum( yht( ky ))*toyr;
	  Mdn(imc,iy,i) = sum(y(kv( ky)))*toyr;
	  Ern(imc,iy,i) = Myn(imc,iy,i) - Mdn(imc,iy,i);
	end
	disp([i imc])
      end
    end
  end

  % get "annual" mean errors
  for i = 1:Nmc
    for j = 1:Nc
      e = squeeze(Ern(i,:,j));
      Ern(i,1,j) = sqrt(e*e'/Ny);
    end
  end
  Ern = squeeze(Ern(:,1,: ));
  Myn = squeeze(mean(Myn,2));
  Mdn = squeeze(mean(Mdn,2));
  
  save absoluteErrors.mat Myv Mdv Erv Myn Mdn Ern
  
end  

load absoluteErrors.mat Erv Ern  

[Nmc,Nc] = size(Ern); 

erv = sort(Erv); erv = erv(round(length(Erv(:,1))*[.15 .25 .5 .75 .85])  );
ern = sort(Ern); ern = ern(round(length(Ern(:,1))*[.15 .25 .5 .75 .85]),:);
ha = 'HorizontalAlignment';
ec = 'EdgeColor';
lw = 'LineWidth';

% errors as percent of the best annual NEE estimate, 2300 kg C/ha/y
% (Goulden et al 2006, Science)
disp('errors as % of the HF annual NEE estimate')
disp(['VPRM = ' int2str(round(100*erv(3)/2250))])
disp(['ANN = ' int2str(round(100*ern(3,:)/2250))])

% scaled error differences assuming applicability to 85e3 km2 (e.g.,
% https://en.wikipedia.org/wiki/Northeastern_coastal_forests) 
dE = median(Ern) - median(Erv*ones(1,Nc)); % kgC/ha/yr
dE = dE*100;      % kgC/km2/yr
dE = dE*85e3;     % kgC/yr for the whole eastern mixed fortest
dE = dE*1e3;      % gC/yr for the whole eastern mixed fortest
dE = dE*1e-12;    % TgC/yr for the whole eastern mixed fortest
disp('error diff. (e_ANN-e_NLR) = ')
disp(sprintf('%6.2f',dE));
disp('TgC/yr for the whole eastern mixed fortest');

cl = [ .6 0 0 ; 0 .6 0 ; .7 .7 .7 ; .8 .4 0 ; 0 .8 .4 ; .4 0 .8 ];

clf

axes('Position',[.3 .5 .4 .4]) 
  
plot([0 8],erv(3)*[1 1]    ,'-',lw,.2,'Color',cl(3,:)); hold on
plot([0 8],erv(3)*[1 1]- 25,'-',lw,.2,'Color',cl(3,:))
plot([0 8],erv(3)*[1 1]- 50,'-',lw,.2,'Color',cl(3,:))
plot([0 8],erv(3)*[1 1]- 75,'-',lw,.2,'Color',cl(3,:))
plot([0 8],erv(3)*[1 1]-100,'-',lw,.2,'Color',cl(3,:))
plot([0 8],erv(3)*[1 1]-125,'-',lw,.2,'Color',cl(3,:))

fill(1+.28*[-1 -1 1 1],erv([2 4 4 2]),'b',ec,'w')
plot([1 1],erv([1 5]),'k-')
plot(1+.12*[-1 1],erv(1)*[1 1],'k-')
plot(1+.12*[-1 1],erv(5)*[1 1],'k-')
scatter(1,erv(3),16,'w','d','filled')
text(1,1251,'VPRM',ha,'c')

for i = 1:Nc
  if rem(i,2)==0; x = i+1-.2; else; x = i+1+.2; end
  fill(x+.28*[-1 -1 1 1],ern([2 4 4 2],i),cl(i,:),ec,'w')
  plot([x x],ern([1 5],i),'k-')
  plot(x+.12*[-1 1],ern(1,i)*[1 1],'k-')
  plot(x+.12*[-1 1],ern(5,i)*[1 1],'k-')
  scatter(x,ern(3,i),16,'w','d','filled')
end

x = .5+[2:2:Nc];
set(gca,'XTick',[1 x],'Box','off','XTickLabel',[])

o = 5:7;
for i = 1:length(x)
  t = [ '{\itN}_p= ' int2str(o(i)) ];
  text(x(i),1251,'ANN',ha,'c')
  text(x(i),1245,  t  ,ha,'c')
end

ylabel('RMS prediction error, kg C (ha\timesyear)^{-1}')

axis([.5 7.5 1255 1415])

set(gcf,'PaperPosition',[.2 .2 8.2 10.6])
print -dpdf -r350 -painters absoluteErrors.pdf

