opts = optimset('display','off');

rng(13456465)

global Ta Prod EVI PAR NEE

load allData PAR Ta rBLUE rNIR rRED NEE rSWIR dy Prod EVI

I{1} = find( dy>331 | dy< 61 ); dr{1,1} = '331- 60';
I{2} = find( dy> 61 & dy<131 ); dr{2,1} =  '61-130';
I{3} = find( dy>131 & dy<236 ); dr{3,1} = '131-235';
I{4} = find( dy>236 & dy<291 ); dr{4,1} = '236-290';
I{5} = find( dy>291 & dy<331 ); dr{5,1} = '291-330';

szL(1) = length(datenum(1998,0,331):datenum(1999,0, 60));
szL(2) = length(datenum(1999,0, 61):datenum(1999,0,130));
szL(3) = length(datenum(1999,0,131):datenum(1999,0,235));
szL(4) = length(datenum(1999,0,236):datenum(1999,0,290));
szL(5) = length(datenum(1999,0,291):datenum(1999,0,330));

for i = 1:length(I)
  dr{i,2} = [ 'n = ' sprintf('%4.1f',1e-3*length(I{i})) 'k' ];
end

Ne = length(I); % number of epochs into which the year is split

%       1   2   3    4     5     6 
X  = [ PAR Ta rBLUE rNIR rRED rSWIR ];
y  = NEE;
Np = length(X(1,:));

% lambda, Topt, PAR, alpha, beta, Tlow. Start from the HF values in
% Mahadevan et al.'s Table 2
p0 = [ .127 20 570 .271 .25 5 ];
e = 0;
while e ~= 1
  [p,~,e] = fminsearch(@dNEE,p0.*(1+randn(1,6)*.05),opts);  
end
disp(p)

a = [ -99 99]; clear p0 opts Prod EVI r* PAR Ta
while a(1)<-50 | a(2)>50
  clear net
  net = feedforwardnet([6 5 4]);
  net.divideFcn = '';
  net.trainParam.showWindow = false;
  net = train(net,X',y');
  yhr = net(X')';
  a = [ min(yhr) max(yhr) ];
end

colormap('default');  clear a yhr
colormap('jet')

v  = -90:5:90;

cn = [ 255 52 52 ]/255; % color for ANN bar
cr = [ 52 52 255 ]/255; % color for nonLinReg bar

un  = 'Units';
clr = 'color';
rt  = 'Rotation';
fs  = 'FontSize';
nr  = 'normalized';
fw  = 'FontWeight';
ha  = 'HorizontalAlignment';

pac =  40:80:2200;  Npc = length(pac);
tac = -15.5:32.5;   Ntc = length(tac);
dp  = .6*(pac(2)-pac(1)); % a slight redundency (overlap)
dt  = .6*(tac(2)-tac(1));
o   = ones(Npc,1);

Da   =     zeros(Npc,Ntc,Ne); % for data availability contour 

NEEn =     zeros(Npc,Ntc,Ne); % for ANN NEEhat
NEEr =     zeros(Npc,Ntc,Ne); % for nonlinear reg NEEhat
NEEd = nan*zeros(Npc,Ntc,Ne); % for data NEE

Df  = zeros(Ne,2);
Dr  = zeros(Ne,2);

ax  = [ tac(1)-2 tac(end)-1 -60 pac(end)-99 ];
cax = [ -35 13 ];

pn  = 'abcd';

clf

for i = 1:Ne % loop on day ranges

  Xh =  X(I{i},:);
  yh =  y(I{i}  );
  mx = median(Xh);
  
  % calculate the data bins
  Xc  = [];
  frm = 9e4*[ 1 -1 1 -1 ]; % a rectangle mask of available observations
  for j = 1:Ntc
    for k = 1:Npc
      Xc  = [ Xc ; pac(k) tac(j) mx(3:end) ];
      l = find( Xh(:,1)>=(pac(k)-dp) & Xh(:,1)<(pac(k)+dp) & ...
	        Xh(:,2)>=(tac(j)-dt) & Xh(:,2)<(tac(j)+dt) );
      if length(l)>5
	NEEd(k,j,i) = mean(yh(l));
	frm(1) = min(frm(1),tac(j)-dt);
	frm(2) = max(frm(2),tac(j)+dt);
	frm(3) = min(frm(3),pac(k)-dp);
	frm(4) = max(frm(4),pac(k)+dp);
      end
    end
  end

  % find the min and max rows (PAR values) w data
  npu = -9;
  npd = 99;
  for j = 1:Npc
    k = find(~isnan(NEEd(j,:,i)));
    if length(k)>1
      npu = max(j,npu);
      npd = min(j,npd);
    end
  end
  
  j0 = ~isnan(NEEd(npd  ,:,i));
  j1 = ~isnan(NEEd(npd+1,:,i));
  j  = find(j0+j1);
  Da(1,min(j):max(j),i) = 1; 
  
  for k = npd+1:npu-1
    j0 = ~isnan(NEEd(k-1,:,i));
    j1 = ~isnan(NEEd(k  ,:,i));
    j2 = ~isnan(NEEd(k+1,:,i));
    j  = find(j0+j1+j2);
    Da(k,min(j):max(j),i) = 1; 
  end

  j0 = ~isnan(NEEd(npu-1,:,i));
  j1 = ~isnan(NEEd(npu  ,:,i));
  j  = find(j0+j1);
  Da(Npc,min(j):max(j),i) = 1; 
  
  % ANN-based y-hat
  NEEn(:,:,i) = reshape(net(Xc')',Npc,Ntc); 

  % nonlineReg y-hat (need to use the rSWIR column)
  NEEr(:,:,i) = reshape(NEEnonLinReg(p,Xc),Npc,Ntc);

  % apply the data availability mask
  j = find( tac<=frm(1) | tac>=frm(2) ); NEEn(:,j,i) = nan; NEEr(:,j,i) = nan;
  j = find( pac<=frm(3) | pac>=frm(4) ); NEEn(j,:,i) = nan; NEEr(j,:,i) = nan;
  
  clear yh Xh j* k l* d

  % plot the ANN predictions
  axes('Position',[.1 1-.133*i .15 .13])
  contourf(tac,pac,NEEn(:,:,i),19,'Color','none'); hold on
  contour(tac,pac,Da(:,:,i),.5*[1 1],'-','Color',.2*[1 1 1],'LineWidth',.4)
  caxis(cax)
  if i==1    ;  title('fANN')
  elseif i==3;  ylabel('PAR, mol m^{-2} s^{-1}'); end
  if i<Ne; set(gca,'XTickLabel',[]); end
  set(gca,'YTick',[1e3 2e3],'YTickLabel',[1 2],'Box','off')
  axis(ax)  
  if i~=2; text(.03,.93,[ '{\bf' pn(1) '}_' int2str(i) ],un,nr);
  else   ; text(.98,.93,[ '{\bf' pn(1) '}_' int2str(i) ],un,nr,ha,'r'); end

  % plot the nonLinReg predictions
  axes('Position',[.255 1-.133*i .15 .13])
  contourf(tac,pac,NEEr(:,:,i),19,'Color','none'); hold on
  contour(tac,pac,Da(:,:,i),.5*[1 1],'-','Color',.2*[1 1 1],'LineWidth',.4)
  caxis(cax)
  if     i==1 ;  title('nonlinear regression')
  elseif i==Ne;  xlabel('air temperature {\itT}, ^\circC'); end
  if i<Ne; set(gca,'XTickLabel',[]); end
  set(gca,'YTickLabel',[],'Box','off')  
  axis(ax)
  if i~=2; text(.03,.93,[ '{\bf' pn(2) '}_' int2str(i) ],un,nr);
  else   ; text(.98,.93,[ '{\bf' pn(2) '}_' int2str(i) ],un,nr,ha,'r'); end

  % plot the data mean NEE
  axes('Position',[.41 1-.133*i .15 .13])
  for j = 1:Ntc
    scatter(tac(j)*o,pac,11,squeeze(NEEd(:,j,i)),'filled'); hold on 
  end
  if     i==1 ;  title('data'); end
  caxis(cax)
  set(gca,'YTickLabel',[],'Box','off')
  if i<Ne; set(gca,'XTickLabel',[]); end
  axis(ax)
  text(.03,.93,[ '{\bf' pn(3) '}_' int2str(i) ],un,nr)
  
  % RMSEs
  axes('Position',[.625 1-.133*i .09 .13])
  need = NEEd(:,:,i); need = need(:); j = find(~isnan(need));
  neen = NEEn(:,:,i); neen = neen(:); neen = neen(j); need = need(j);
  neer = NEEr(:,:,i); neer = neer(:); neer = neer(j);
  Dr(i,1) = (need-neen)'*(need-neen);
  Dr(i,2) = (need-neer)'*(need-neer);
  j = (need-mean(need))'*(need-mean(need));
  Df(i,:) = Dr(i,:)/j;
  fill(  [.10 .10 .93 .93],Df(i,1)*[0 1 1 0],cn,'EdgeColor',cn); hold on
  fill(1+[.07 .07 .90 .90],Df(i,2)*[0 1 1 0],cr,'EdgeColor',cr)
  set(gca,'XTick',[],'Box','off')
  if i==3
    ylabel('normalized mean squared error')
  end
  if i==1; title('fit errors');  end
  axh = [ 0 2 0 max(.32,max(Df(i,:))*1.05) ];
  axis(axh)
  j = int2str(round(100*min(Df(i,:))/max(Df(i,:))));
  j = [j '%'];
  if Df(i,1)>Df(i,2)
    clh = cr; xh = 1.24; yh = Df(i,2)+axh(4)*.08; cc = cr;
  else              
    clh = cn; xh = 0.24; yh = Df(i,1)+axh(4)*.08; cc = cn;
  end
  if min(Df(i,:))>(.2*max(Df(i,:))); yh = axh(4)*.13; cc = 'w'; end 
  text(xh,yh,j,'Color',cc,'FontSize',9)
  text(.06,.9,[ '{\bf' pn(4) '}_' int2str(i) ],un,nr)
  j = find(dr{i,1}=='-');
  text(1.03,.93,   'days'      ,un,nr)
  text(1.03,.83,dr{i,1}(1: j ) ,un,nr)
  text(1.03,.73,dr{i,1}(j:end) ,un,nr)
  text(1.03,.19,dr{i,2}(1: 3 ) ,un,nr)
  text(1.03,.09,      '= '     ,un,nr)
  j = length(dr{i,2}(5:end));
  j = (j>4)*.043;
  text(1.08+j,.09,dr{i,2}(5:end),un,nr)
  dr1 = sqrt(Dr(i,1)/length(need));
  dr2 = sqrt(Dr(i,2)/length(need));
  dfr = dr2-dr1;                     % error diff,   micromol/m2/s
  dfr = dfr*1e-6;                    % error diff,        mol/m2/s
  dfr = dfr*1e4 ;                    % error diff,        mol/ha/s
  dfr = dfr*12e-6;                   % error diff, metricTonC/ha/s
  dfr = dfr*3600*24*szL(i);          % error diff, metricTonC/ha/season
  text(1.03,.55,   '\Deltae ='      ,un,nr,clr,[120 0 120]/255)
  text(1.03,.45,[ '= ' sprintf('%3.1f',dfr)],un,nr,clr,[120 0 120]/255)
  if     i==1; text(2.2,.56,'{\bflate}'  ,un,nr,fs,13,ha,'c');
               text(2.2,.43,'{\bfwinter}',un,nr,fs,13,ha,'c');
  elseif i==2; text(2.2,.5 ,'{\bfspring}',un,nr,fs,13,ha,'c');
  elseif i==3; text(2.2,.5 ,'{\bfsummer}',un,nr,fs,13,ha,'c');
  elseif i==4; text(2.2,.5 ,'{\bffall}'  ,un,nr,fs,13,ha,'c');
  elseif i==5; text(2.2,.56,'{\bfearly}' ,un,nr,fs,13,ha,'c');
               text(2.2,.43,'{\bfwinter}',un,nr,fs,13,ha,'c'); end
end
text(0.55,-.2,'fANN' ,clr,cn,ha,'c')
text(1.55,-.2,'NLR',clr,cr,ha,'c')

% colorbar
axes('Position',[.07 .261 .72 .023])
i = linspace(cax(1),cax(2),19);
contourf(i,1:2,[i;i],19,'Color','none')
set(gca,'XTick',-30:5:30,'YTick',[])
xlabel('NEE (C_\uparrow flux), \mumol m^2 s^{-1}')

set(gcf,'PaperPosition',[.2 .2 8.2 10.6])
print -dpdf -r350 -painters monthlyCalculatedNEE.pdf

