opts = optimset('display','off');

rng(13456465)

global Ta Prod EVI PAR NEE

load allData PAR Ta rBLUE rNIR rRED NEE rSWIR dy Prod EVI

I    = cell(2,1);
I{1} = find( (dy>291 | dy<131) & PAR>250 ); dr{1,1} = '291-130';
I{2} = find(  dy>131 & dy<291  & PAR>250 ); dr{2,1} = '131-290';

%        1   2   3    4     5     6 
X   = [ PAR Ta rBLUE rNIR rRED rSWIR ];
y   = NEE;
Np  = length(X(1,:));
Nt  = length(X(:,1));
Ne  = length(I);        % number of doy epochs
Nmc = 100;              % MC realization number
oh  = ones(1,Nmc);

if exist('Tdependence2.mat')~=2

  Ta0   =   Ta;
  Prod0 = Prod;
  EVI0  =  EVI; 
  PAR0  =  PAR;
  NEE0  =  NEE;
  
  % lambda, Topt, PAR, alpha, beta, Tlow. Start from the HF values in
  % Mahadevan et al.'s Table 2
  p0  = [ .127 20 570 .271 .25 5 ];
  Prm = zeros(Nmc,6);
  Net =  cell(Nmc,1);
  
  nmc = 0;
    
  while nmc<Nmc

    k = randperm(Nt,3e4);

    Ta   =   Ta0(k);
    Prod = Prod0(k);
    EVI  =  EVI0(k); 
    PAR  =  PAR0(k);
    NEE  =  NEE0(k);

    [p,~,e] = fminsearch(@dNEE,p0,opts); 
    if e==1
      disp(p)
      yht = NEEnonLinReg(p,X);
      clear net
      net = feedforwardnet([6 5 4]);
      net.divideFcn = '';
      net.trainParam.showWindow = false;
      net = train(net,X(k,:)',y(k)');
      yht = [ yht' net(X') ];

      if max(abs(yht))<51 
	nmc        = nmc + 1;
	Prm(nmc,:) = p;
	Net{nmc}   = net;        disp(nmc)
      end
      
    end
    
  end
  
  save Tdependence2.mat Prm Net I
  
else
  
  load Tdependence2.mat Prm
  
end

load allData Prod EVI PAR

% first, T-independent terms

T1 = zeros(Ne,1); % for stats of the compound product term in GEE
for i = 1:Ne

  ov = ones(length(I{i}),1);

  % get the compound GEE term
  %           -lambda        x  Psclae x Wscale x       EVI   
  t1    = -(ov*Prm(:,1)') .* (Prod(I{i})*oh) .* (EVI(I{i})*oh);

  % multiply by the PAR factor
  t2    = PAR(I{i});
  t2    = (t2*oh) ./ (1+(t2*oh).*(ov*(1./Prm(:,3))' ));
  t1    = t1.*t2;

  % get the median GEE/Tscale
  t1    = sort(t1(find(t1)),'descend');
  T1(i) = t1(round(length(t1)*.5));
   
end

clear oh ov t1 t2 r* Prod y EVI opts dy; load Tdependence2 Net

% next, the T-dependent respiration and Tscale

Tg   = zeros(25,Ne); % for the epoch-specific time grids
Rsp  = zeros(25,Ne); % for respiration
GEE  = zeros(25,Ne); % for GEE

Tmn =   0;
Tmx =  40;

Mn = zeros(25,Ne  ); % for mean  observed NEE
Sn = zeros(25,Ne,2); % for stdev observed NEE

Ma = zeros(25,Ne); % for 50th % ANN-based predicted NEE
mPAR = zeros(25,Ne);
mp   = zeros( 1,Ne);
Npts = zeros(25,Ne);

for i = 1:Ne
  
  % the T grid for the season
  tg       = X(I{i},2);
  tg       = sort(tg);
  tg       = tg(round(length(tg)*[.025 .975]));
  tg       = linspace(tg(1),tg(2),25)'; 
  Tg(:,i)  = tg;
  mp(i)    = median(PAR(I{i}));
  
  % get epoch-mean NEE and ANNyht dist
  for j = 1:25
    k  = find( Ta(I{i})>(tg(j)-1.6) & Ta(I{i})<(tg(j)+1.6) );
    mPAR(j,i) = median(PAR(I{i}(k)));
    Npts(j,i) = length(k);
    
    y  = zeros(length(k),Nmc);
    % observed NEE
    Mn(j,i)   = median(NEE(I{i}(k)));
    l         =  sort(NEE(I{i}(k)));
    Sn(j,i,:) =  l(round(length(l)*[.3 .7]));
    % ANN-based NEE prediction
    for l = 1:Nmc
      y(:,l) = Net{l}(X(I{i}(k),:)')';
    end
    y = sort(y(:));
    Ma(j,i) = y(round(length(y)*.5));
  end
  mPAR(:,i) = conv(mPAR(:,i),[1 2 1]/4,'same');
  
  % respiration
  j        = max(tg,Prm(:,6)');
  j        = (ones(25,1)*Prm(:,4)') .* j;
  j        = j + (ones(25,1)*Prm(:,5)');
  j        = sort(j');
  Rsp(:,i) = j(round(Nmc*.5),:)'; 
  
  % Tscale with the temp grid
  Tscl     = (max(Tmn,tg) - Tmn) .* (tg - Tmx);
  Tscl     = Tscl*ones(1,Nmc);
  j        = (tg*ones(1,Nmc)) - (ones(25,1)*Prm(:,2)');
  Tscl     = Tscl ./ ( Tscl - j.*j);
  Tscl     = Tscl .* T1(i); % multiply by the other terms
  Tscl     = sort(Tscl');

  % scale T_scale by the rest of the multiplicative GEE factor
  GEE(:,i) = Tscl(round(Nmc*.5),:)';
 
end

Mpts = Npts - min(Npts(:)); 
Mpts = 1+Mpts/max(Mpts(:));

clear PAR NEE Tscl r* EVI Net y i opts Prod T1 Nmc Np
clear Th* Yh* j p net tg Tmn Tmx Prm Xh Ta X I Nt mX

un  = 'Units';
clr = 'color';
rt  = 'Rotation';
fs  = 'FontSize';
lw  = 'LineWidth';
fw  = 'FontWeight';
nr  = 'normalized';
ha  = 'HorizontalAlignment';

ax  = [      min(Tg(:,1))-1 max(Tg(:,1))+1 ]; % x-range
ax  = [ ax ; min(Tg(:,2))-1 max(Tg(:,2))+1 ]; % x-range

cax = [ min(mPAR(:)) max(mPAR(:)) ];

pn = 'abcdefghijklm';

clf

for i = Ne:-1:1
  
  axes('Position',[.07+(i-1)*.165 .4 .16 .25])
  plot(39*[-1 1],[0 0],'-','Color',.8*[1 1 1]); hold on
  h(1) = plot(Tg(:,i),GEE(:,i)+Rsp(:,i),'r-');
  h(2) = plot(Tg(:,i),GEE(:,i),'b-');
  h(3) = plot(Tg(:,i),Rsp(:,i),'m-' );
  h(4) = plot(Tg(:,i), Ma(:,i),' -','Color',[.7 .4 .1] );
  h(5) = scatter(Tg(:,i),Mn(:,i),3,'g','filled');
  for j = 1:25
    plot(Tg(j,i)*[1 1],squeeze(Sn(j,i,:)),'g-',lw,.2)
    plot(Tg(j,i)+.2*[-1 1],squeeze(Sn(j,i,2))*[1 1],'g-',lw,.2)
    plot(Tg(j,i)+.2*[-1 1],squeeze(Sn(j,i,1))*[1 1],'g-',lw,.2)
  end
  axis([ax(i,:) -24 10])
  if i>1
    set(gca,'YTicklabel',[])
  else
    xlabel('                             air temperature, ^\circC')
    ylabel('C \uparrow flux, \mumol m^2 s^{-1}')
    legend(h,'NEE_{50}','GEE_{50}','{\itR}_{50}', ...
	'NEE_{50}^{ANN}','NEE_{50}^{obs}','Location','South')
  end
  text(.5,1.04,[ 'days ' dr{i} ],un,nr,ha,'c')
  text(.03,.95,pn(i),un,nr,fs,15,fw,'b')

  % plot the mean PAR and the number of pts of each bin
  clm = linspace(min(mPAR(:,i)),max(mPAR(:,i)),25);
  clm = [ clm ; clm ];
  axes('Position',[.07+(i-1)*.165 .31 .16 .04])
  contourf(clm(1,:),[1 2],clm,'EdgeColor','w');   hold on
  plot(clm(1,:),Mpts(:,i),'k-')
  j = 1+([ 500 1500 ]-min(Npts(:)))/(max(Npts(:))-min(Npts(:)));
  plot(mp(i)*[1 1],[1 2],'k-',lw,1)
  set(gca,'YTick',[],'YLim',[1 2],'XTick',600:200:2e3)
  set(gca,'XTick',600:200:2e3,'YColor','w','Box','off')
  set(gca,'YTick',j,'YTicklabel',[5 15])
  if i==1; ylabel('bin {\itN}/100'); else; set(gca,'YTickLabel',[]); end
  caxis(cax)
  axis([ clm(1,[1 end]) 1 2])
  text(.89,.84,pn(2+i),un,nr,fs,15,fw,'b')
  
end

xlabel('                               median bin PAR, \mumol m^{-2} s^{-1}')

set(gcf,'PaperPosition',[.2 .2 8.2 10.6])
print -dpdf -r350 -painters Tdependence2.pdf

% seasonal RMSEs

oW = Mn(:,1); % winter obvservations
oS = Mn(:,2); % summer obvservations

vW = GEE(:,1)+Rsp(:,1); % winter VPRM prediction
vS = GEE(:,2)+Rsp(:,2); % summer VPRM prediction

nW = Ma(:,1); % winter fANN prediction
nS = Ma(:,2); % summer fANN prediction

rmseNw = sqrt( (oW-nW)'*(oW-nW)/25);
rmseNs = sqrt( (oS-nS)'*(oS-nS)/25);

rmseVw = sqrt( (oW-vW)'*(oW-vW)/25);
rmseVs = sqrt( (oS-vS)'*(oS-vS)/25);

dW = rmseVw - rmseNw; % difference in micromol/(m2 s)
dS = rmseVs - rmseNs; % difference in micromol/(m2 s)

dW = dW*1e-6; % difference in mol/(m2 s)
dS = dS*1e-6; % difference in mol/(m2 s)

dW = dW*1e4; % difference in mol/(ha s)
dS = dS*1e4; % difference in mol/(ha s)

dW = dW*12; % difference in gC/(ha s)
dS = dS*12; % difference in gC/(ha s)

% differences in gC/(ha season)
dW = dW*3600*24*length(datenum(1991,0,291):datenum(1992,0,130));
dS = dS*3600*24*length(datenum(1991,0,131):datenum(1991,0,290));

dW = dW*1e-3;% winter difference in kgC/(ha season)
dS = dS*1e-3;% summer difference in kgC/(ha season)

disp('winter errors: (fANN, VPRM, diff)')
disp([ rmseNw rmseVw dW ])

disp('summer errors: (fANN VPRM, diff)')
disp([ rmseNs rmseVs dS ])

