% calculate total C uptake over the period cover by the observation
% using (1) the best ANN; and (2) the nonLinReg

if exist('totalCuptake.mat')~=2
  
  rng(4321234)
  
  global Ta Prod EVI PAR NEE

  opts = optimset('display','off');
  
  load allData PAR NEE Ta r* Prod EVI
  
  PAR = max(0,PAR);
  
  Nt  = length(NEE);
  
  kt  = randperm(Nt,round(Nt*.5)); % training set
  kv  = find(~ismember(1:Nt,kt )); % validation set
  
  Prodo = Prod;	  Prod = Prod(kt);	
  EVIo  = EVI; 	  EVI  =  EVI(kt); 	
  PARo  = PAR; 	  PAR  =  PAR(kt); 	
  NEEo  = NEE;    NEE  =  NEE(kt);  
  Tao   = Ta;     Ta   =   Ta(kt);	
  
  % lambda, Topt, PAR, alpha, beta. Start from the HF values in
  % Mahadevan et al.'s Table 2, where the elements of p are:
  % (1) lambda; (2) Topt; (3) PAR0; (4) alpha; (5) beta; (6) Tlow  

  e = 0;
  while e ~= 1
    i = [.127 20 570 .271 .25 5].*(1+randn(1,6)*.05);
    [p,~,e] = fminsearch(@dNEE,i,opts);  
  end

  % load soil temp
  load Tsoil Tsoil
  
  % add the antecedent precip predictor  
  load ../Data/antecedentPrecipPredictor.mat aPp

  %       1    2    3    4     5     6   7    8
  X  = [ PARo Tao rBLUE rNIR rRED rSWIR aPp Tsoil ];
  y  = NEEo;
  Np = length(X(1,:)); clear opts *o r* NEE
  
  yht = NEEnonLinReg(p,X(kv,1:6));

  clear net
  net = feedforwardnet([6 5 4]);
  net1.divideFcn = '';
  net.trainParam.showWindow = false;
  net = train(net,X(kt,1:6)',y(kt)');
  yht = [ yht net(X(kv,1:6)')' ];

  clear net1
  net1 = feedforwardnet([6 5 4]);
  net1.divideFcn = '';
  net1.trainParam.showWindow = false;
  net1 = train(net1,X(kt,[1 2 4 6 8])',y(kt)');
  yht  = [ yht net1(X(kv,[1 2 4 6 8])')' ];

  R2  = zeros(200,3);
  for i = 1:200
    j   = randperm(length(kv),1e3);
    k   = kv(j);
    for m = 1:3
      r2  = cor4vects(yht(j,m),y(k));  R2(i,m)  = r2*r2;
    end
  end
  R2  = [ mean( R2) ; std( R2) ]; 
  disp( R2)
  
  save totalCuptake.mat net net1 p yht* kv kt R2
  
end

load totalCuptake.mat
  
% test significance of R2s
C   = [];
Nmc = 200; % make SURE this is the same as above
for i = 1:length(R2(1,:))-1
  for j = i+1:length(R2(1,:))
    s  = sqrt( (Nmc-1)*(R2(1,i)^2 + R2(1,j)^2)/(2*Nmc-2) );
    se = s * sqrt( 2/Nmc );
    t = (R2(1,j) - R2(1,i))/se;
    p = 1-tcdf(t,2*Nmc-2);
    % use one-sided test appropriate for H1: R2_j > R2_i
    C = [ C ; i j t p ];
  end
end
   
load allData NEE t

clf

[m,mi] = max(NEE(kv(1:1900)));

% text(-38,19,[ '{\itR}^2_{AUG} = ' sprintf('%4.2f',R2(1,1))])

% left panels

axes('Position',[.08 .55 .3 .4])
plot(NEE(kv),yht(:,1),'.b',[-50 50],[-50 50],'m-')
axis([-40 39 -29 22])
xlabel('observed NEE')
ylabel('predicted NEE')
title('{\bfnonlinear regression}')
text(-38,20,'{\bfa}','FontSize',14)
i = [ '{\itR}_{cv}^{ 2} \approx ' sprintf('%5.3f',R2(1,1)) ];
i = [ i ' \pm ' sprintf('%5.3f',R2(2,1)) ];
text(.99,.06,i,'Units','normalized','HorizontalAlignment','r')

axes('Position',[.08 .1 .3 .36])
plot(t(kv),NEE(kv),'.g',t(kv),yht(:,1),'.b')
xlabel('calendar year')
ylabel('NEE')
axis([t(1)-.1 t(kv(end))+.1 -40 39])
text(t(kv(mi))+.2,m,'observed NEE','Color','g')
text(t(1)+.2,35,'{\bfb}','FontSize',14)

% middle panels

axes('Position',[.384 .55 .3 .4])
plot(NEE(kv),yht(:,2),'.c',[-50 50],[-50 50],'m-')
axis([-40 39 -29 22])
xlabel('observed NEE')
set(gca,'YTickLabel',[])
title('{\bfANN_t(6,5,4)}')
text(-38,20,'{\bfc}','FontSize',14)
i = [ '{\itR}_{cv}^{ 2} \approx ' sprintf('%5.3f',R2(1,2)) ];
i = [ i ' \pm ' sprintf('%5.3f',R2(2,2)) ];
text(.99,.06,i,'Units','normalized','HorizontalAlignment','r')

axes('Position',[.384 .1 .3 .36])
plot(t(kv),NEE(kv),'.g',t(kv),yht(:,2),'.c')
xlabel('calendar year')
axis([t(1)-.1 t(kv(end))+.1 -40 39])
text(t(kv(mi))+.2,m,'observed NEE','Color','g')
set(gca,'YTickLabel',[])
text(t(1)+.2,35,'{\bfd}','FontSize',14)

% right panels

axes('Position',[.689 .55 .3 .4])
plot(NEE(kv),yht(:,3),'.r',[-50 50],[-50 50],'b-')
axis([-40 39 -29 22])
xlabel('observed NEE')
set(gca,'YTickLabel',[])
title('{\bfANN_a(6,5,4)}')
text(-38,20,'{\bfe}','FontSize',14)
i = [ '{\itR}_{cv}^{ 2} \approx ' sprintf('%5.3f',R2(1,3)) ];
i = [ i ' \pm ' sprintf('%5.3f',R2(2,3)) ];
text(.99,.06,i,'Units','normalized','HorizontalAlignment','r')

axes('Position',[.689 .1 .3 .36])
plot(t(kv),NEE(kv),'.g',t(kv),yht(:,3),'.r')
xlabel('calendar year')
axis([t(1)-.1 t(kv(end))+.1 -40 39])
text(t(kv(mi))+.2,m,'observed NEE','Color','g')
set(gca,'YTickLabel',[])
text(t(1)+.2,35,'{\bff}','FontSize',14)

set(gcf,'PaperPosition','default')
print -dpdf -r350 -painters totalCuptake.pdf

% now calculate the file's namesake, meaning cummulative total C uptake

load totalCuptake net p
  
load allData PAR NEE Ta r* Prod EVI t

% load soil temp
load Tsoil Tsoil
  
% add the antecedent precip predictor  
load ../Data/antecedentPrecipPredictor.mat aPp

%                  1   2   3    4     5     6   7    8
X       = [ max(0,PAR) Ta rBLUE rNIR rRED rSWIR aPp Tsoil ];
y       = NEE;
yhtNLR  = NEEnonLinReg(p,X);
yhtANNt = net(X(:,1:6)')';         clear PAR Prod EVI Ta r* NEE kt kv m*
yhtANNa = net1(X(:,[1 2 4 6 8])')'; clear net p  

f      =   12e-6; % 1e-6 mol CO2/m2/s  ->  g C/m2/s
f      = -f*3600; % g C/m2/s upward    ->  g C/m2/hr downward
f      =  f*1e-3; % g C/m2/hr downward -> kg C/m2/hr downward 

% the integral dt is in kg C/m2
cumCupObs  = cumsum(y      *f);
cumCupANNt = cumsum(yhtANNt*f);
cumCupANNa = cumsum(yhtANNa*f);
cumCupNLR  = cumsum(yhtNLR *f); tc = t-2e3;

RMSEt = cumCupANNt - cumCupObs;
RMSEa = cumCupANNa - cumCupObs;
RMSEr = cumCupNLR  - cumCupObs;
RMSEt = RMSEt' * RMSEt;
RMSEa =	RMSEa' * RMSEa; 
RMSEr =	RMSEr' * RMSEr;
RMSEt = sqrt(RMSEt/length(cumCupObs));
RMSEa =	sqrt(RMSEa/length(cumCupObs));
RMSEr =	sqrt(RMSEr/length(cumCupObs));

clf

axes('Position',[.45 .5 .32 .36])
plot(tc,cumCupObs ,'g.','MarkerSize',2); hold on
plot(tc,cumCupANNt,'b.','MarkerSize',2)
plot(tc,cumCupANNa,'m.','MarkerSize',2)
plot(tc,cumCupNLR ,'r.','MarkerSize',2)
xlabel('years of C accumulation since 1/1/2000')
ylabel('kg C m^{-2}')
axis([tc(1)-.3 tc(end)+.3 min(cumCupNLR)-.035 max(cumCupObs)+.035])
set(gca,'XTick',0:2:20)
text(tc(290),3.01,'observations','Color','g')  
text(tc(290),2.76,'augmented neural net','Color','m')  
text(tc(290),2.51,'full neural net','Color','b')  
text(tc(290),2.26,'nonlinear regression','Color','r')  
text(tc(290),1.95,'{\bfRMSEs:}')  
plot(tc(290)+[0 3.2],1.8*[1 1],'k-')
text(.9,1.65,sprintf('%4.2f',RMSEa),'Color','m')  
text(.9,1.40,sprintf('%4.2f',RMSEt),'Color','b')  
text(.9,1.15,sprintf('%4.2f',RMSEr),'Color','r')  
tm = get(gca,'YTick');
for i = 1:length(tm)
  th = tm(i)*1e04; %         kg C/hectare
  th = th   *1e-3; % metric ton CO2/hectare
  th = int2str(th);
  text(tc(end)+.7,tm(i),th)
end
text(1.15,.5,'metric ton C ha^{-1}', ...
    'HorizontalAlignment','c', ...
    'Rotation',90,'Units','normalized')
text(.5,1.05,'Harvard Forest cumulative carbon uptake', ...
    'HorizontalAlignment','c','Units','normalized')

print -dpdf -r350 -painters totalCuptake2.pdf
