% read, quality-assure and interpolate all data 
load ../Data/allDatFiles

yr = ta(:,1); % year
nd = ta(:,2); % nominal  day-of-year
md = ta(:,3); % averaged day-of-year

% if the actual averaged doy is within reason of the file name (up to
% the nominal frequency of an 8-day return time), we are OK
i = find( abs(md-nd)>=7.5 | (md-nd)<-1 ); 
md(i) = nd(i)+4; % assume mean half way pt (4 days) later

mxd = zeros(length(yr),1);
for i = 1:length(yr)
  j = calendar(yr(i),2); % Feb
  j = max(j);  
  mxd(i) = 365 + max(j) - 28;
end

% satellite data time grid
ts = datenum([ yr yr*0 md ]); 
ts  = yr + (md-.5)./mxd;  clear md nd ta yr hr HFdat

% load HF EMS tower data up to 2015
load ../Data/HF_9215_filled.txt
HFdat = HF_9215_filled; clear HF_9215_filled

yr  = HFdat(:,1); % year
dy  = HFdat(:,3); % day of year
hr  = HFdat(:,4); % hour EST

mxd = zeros(length(yr),1);
for i = 1:length(yr)
  j = calendar(yr(i),2); % Feb
  j = max(j);  
  mxd(i) = 365 + max(j) - 28;
end

td  = yr + (dy-1+hr/24)./mxd; clear yr dy hr

NEE = HFdat(:, 7); % obs NEE
ust = HFdat(:,10); % u_star
Ta  = HFdat(:,16); % obs Ta 27
Taf = HFdat(:,17); % filles Ta 27
PAR = HFdat(:,28); % obs par28

i   = find(ust<.22); NEE(i) = -999.99; clear ust HFdat

% fill Ta27.obs with filled values between mid-2009 and 2012
i   = find( td>2009.4 & td<2012 & Ta==-999.99 & Taf~=-999.99 );
Ta(i) = Taf(i);  clear Taf

% keep only full data time pts in the HF observe hourly data
i   = find( NEE~=-999.99 & Ta~=-999.99 & PAR~=-999.99 );
td  = td(i);
NEE = NEE(i);
Ta  = Ta(i);
PAR = PAR(i);

% nominal rez: td ~ hourly, ts ~ 8 day 
i   = find( td>=ts(1) & td<=ts(end) );
t   = td(i);
NEE = NEE(i);
Ta  = Ta(i);
PAR = PAR(i);

Ra  = interp1(ts,Ra,t);  %clear i t0 te td ts

rRED  = Ra(:,1); % length(find(isnan(rRED))) = 94
rNIR  = Ra(:,2);
rBLUE = Ra(:,3);
rSWIR = Ra(:,4);  clear Ra i ts td

save allData4NeuralNetwork.mat r* NEE Ta PAR t